@@ -1,8 +1,11 @@
 """ Shared utilities: logging """
-import logging
-import os
 
-__all__ = ["DEBUG", "get_logger", "init_logger"]
+import os
+import re
+import logging
+from dataclasses import dataclass
+
+__all__ = ["DEBUG", "get_logger", "state", "PyprError", "apply_variables"]
 
 DEBUG = os.environ.get("DEBUG", False)
 
@@ -57,8 +60,13 @@ def init_logger(filename=None, force_debug=False):
     LogObjects.handlers.append(stream_handler)
 
 
-def get_logger(name="pypr", level=None):
-    "Returns a logger for `name`"
+def get_logger(name="pypr", level=None) -> logging.Logger:
+    """Returns a named logger
+
+    Args:
+        name (str): logger's name
+        level (int): logger's level (auto if not set)
+    """
     logger = logging.getLogger(name)
     if level is None:
         logger.setLevel(logging.DEBUG if DEBUG else logging.WARNING)
@@ -69,3 +77,46 @@ def get_logger(name="pypr", level=None):
         logger.addHandler(handler)
     logger.info('Logger "%s" initialized', name)
     return logger
+
+
+@dataclass
+class SharedState:
+    "Stores commonly requested properties"
+    active_workspace: str = ""  # workspace name
+    active_monitor: str = ""  # monitor name
+    active_window: str = ""  # window address
+
+
+state = SharedState()
+"""
+Exposes most-commonly accessed attributes to avoid specific IPC requests
+- `active_monitor` monitor's name
+- `active_workspace` workspace's name
+- `active_window` window's address
+"""
+
+
+def apply_variables(template: str, variables: dict[str, str]):
+    """Replace [var_name] with content from supplied variables
+    Args:
+        template: the string template
+        variables: a dict containing the variables to replace
+    """
+    pattern = r"\[([^\[\]]+)\]"
+
+    def replace(match):
+        var_name = match.group(1)
+        return variables.get(var_name, match.group(0))
+
+    return re.sub(pattern, replace, template)
+
+
+def apply_filter(text, filt_cmd: str):
+    """Apply filters to text
+    Currently supports only "s" command fom vim/ed"""
+    if not filt_cmd:
+        return text
+    if filt_cmd[0] == "s":  # vi-like substitute
+        (_, base, replacement, opts) = filt_cmd.split(filt_cmd[1])
+        return re.sub(base, replacement, text, count=0 if "g" in opts else 1)
+    return text
