@@ -1,8 +1,8 @@
 """ expose Brings every client window to screen for selection
 """
-from typing import Any, cast
 
 from .interface import Plugin
+from ..common import state
 
 
 class Extension(Plugin):  # pylint: disable=missing-class-docstring
@@ -19,8 +19,6 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring
         """Expose every client on the active workspace.
         If expose is active restores everything and move to the focused window"""
         if self.exposed:
-            aw: dict[str, Any] = cast(dict, await self.hyprctlJSON("activewindow"))
-            focused_addr = aw["address"]
             commands = []
             for client in self.exposed_clients:
                 commands.append(
@@ -29,18 +27,13 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring
             commands.extend(
                 [
                     "togglespecialworkspace exposed",
-                    f"focuswindow address:{focused_addr}",
+                    f"focuswindow address:{state.active_window}",
                 ]
             )
             await self.hyprctl(commands)
             self.exposed = []
         else:
-            cur_wrkspc = await self.hyprctlJSON("activeworkspace")
-            self.exposed = [
-                c
-                for c in cast(list, await self.hyprctlJSON("clients"))
-                if c["workspace"]["id"] != cur_wrkspc["id"]
-            ]
+            self.exposed = await self.get_clients(workspace_bl=state.active_workspace)
             commands = []
             for client in self.exposed_clients:
                 commands.append(
