@@ -1,18 +1,22 @@
 " Scratchpads addon "
 import os
-import logging
 import time
+import logging
 import asyncio
-import subprocess
-from typing import Any, cast
+from typing import Any, cast, Callable
 from functools import partial
 from collections import defaultdict
 
+from aiofiles import os as aios
+from aiofiles import open as aiopen
+
 from ..ipc import notify_error, get_client_props, get_focused_monitor_props
 from .interface import Plugin
+from ..common import state
 
 DEFAULT_MARGIN = 60  # in pixels
 AFTER_SHOW_INHIBITION = 0.2  # 200ms of ignorance after a show
+DEFAULT_HYSTERESIS = 0.4  # In seconds
 
 # Helper functions {{{
 
@@ -128,6 +132,7 @@ class Animations:  # {{{
 class Scratch:  # {{{
     "A scratchpad state including configuration & client state"
     log = logging.getLogger("scratch")
+    get_client_props: Callable
 
     def __init__(self, uid, opts):
         self.uid = uid
@@ -142,6 +147,7 @@ class Scratch:  # {{{
         self.initialized = False
         self.meta = {}
         self.space_identifier = None
+        self.monitor = ""
 
     async def initialize(self, ex):
         "Initialize the scratchpad"
@@ -157,12 +163,16 @@ class Scratch:  # {{{
         "is the process running ?"
         if self.conf.get("process_tracking", True):
             path = f"/proc/{self.pid}"
-            if os.path.exists(path):
-                with open(os.path.join(path, "status"), "r", encoding="utf-8") as f:
-                    for line in f.readlines():
+            if await aios.path.exists(path):
+                async with aiopen(
+                    os.path.join(path, "status"), mode="r", encoding="utf-8"
+                ) as f:
+                    for line in await f.readlines():
                         if line.startswith("State"):
-                            state = line.split()[1]
-                            return state not in "ZX"  # not "Z (zombie)"or "X (dead)"
+                            proc_state = line.split()[1]
+                            return (
+                                proc_state not in "ZX"
+                            )  # not "Z (zombie)"or "X (dead)"
         else:
             if getattr(self, "bogus_pid", False):
                 return bool(await self.get_client_props(cls=self.conf["class"]))
@@ -215,21 +225,21 @@ class ScratchDB:  # {{{
     _states: defaultdict[str, set[Scratch]] = defaultdict(set)
 
     # State management {{{
-    def getByState(self, state: str):
-        "get a set of `Scratch` being in `state`"
-        return self._states[state]
+    def getByState(self, status: str):
+        "get a set of `Scratch` being in `status`"
+        return self._states[status]
 
-    def hasState(self, scratch: Scratch, state: str):
-        "Returns true if `scratch` has state `state`"
-        return scratch in self._states[state]
+    def hasState(self, scratch: Scratch, status: str):
+        "Returns true if `scratch` has state `status`"
+        return scratch in self._states[status]
 
-    def setState(self, scratch: Scratch, state: str):
-        "Sets `scratch` in the provided state"
-        self._states[state].add(scratch)
+    def setState(self, scratch: Scratch, status: str):
+        "Sets `scratch` in the provided status"
+        self._states[status].add(scratch)
 
-    def clearState(self, scratch: Scratch, state: str):
-        "Unsets the the provided state from the scratch"
-        self._states[state].remove(scratch)
+    def clearState(self, scratch: Scratch, status: str):
+        "Unsets the the provided status from the scratch"
+        self._states[status].remove(scratch)
 
     # }}}
 
@@ -277,7 +287,7 @@ class ScratchDB:  # {{{
             self._by_addr[scratch.address] = scratch
         else:
             if name is not None:
-                d = self._by_name
+                d: dict[Any, Scratch] = cast(dict[str, Scratch], self._by_name)
                 v = name
             elif pid is not None:
                 d = self._by_pid
@@ -288,7 +298,7 @@ class ScratchDB:  # {{{
             d[v] = scratch
         # }}}
 
-    def get(self, name=None, pid=None, addr=None) -> Scratch:
+    def get(self, name=None, pid=None, addr=None) -> Scratch | None:
         "return the Scratch matching given name, pid or address"
         # {{{
         assert 1 == len(list(filter(bool, (name, pid, addr)))), (
@@ -297,7 +307,7 @@ class ScratchDB:  # {{{
             addr,
         )
         if name is not None:
-            d = self._by_name
+            d: dict[Any, Scratch] = self._by_name
             v = name
         elif pid is not None:
             d = self._by_pid
@@ -313,71 +323,71 @@ class ScratchDB:  # {{{
 
 
 class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
-    procs: dict[str, subprocess.Popen] = {}
+    procs: dict[str, asyncio.subprocess.Process] = {}  # pylint: disable=no-member
     scratches = ScratchDB()
 
-    focused_window_tracking: dict[str, dict] = {}
+    focused_window_tracking: dict[str, str] = {}
 
     workspace = ""  # Currently active workspace
-    monitor = ""  # CUrrently active monitor
+    monitor = ""  # Currently active monitor
+
+    _hysteresis_tasks: dict[str, asyncio.Task]  # non-blocking tasks
 
     def __init__(self, name):
         super().__init__(name)
+        self._hysteresis_tasks = {}
         self.get_client_props = partial(get_client_props, logger=self.log)
         Scratch.get_client_props = self.get_client_props
         self.get_focused_monitor_props = partial(
             get_focused_monitor_props, logger=self.log
         )
 
-    async def init(self):
-        "Initializes the Scratchpad extension"
-        self.workspace = (await self.hyprctlJSON("activeworkspace"))["name"]
-        self.monitor = next(
-            mon for mon in (await self.hyprctlJSON("monitors")) if mon["focused"]
-        )
-
     async def exit(self) -> None:
         "exit hook"
 
         async def die_in_piece(scratch: Scratch):
-            proc = self.procs[scratch.uid]
-            proc.terminate()
-            for _ in range(10):
-                if not await scratch.isAlive():
-                    break
-                await asyncio.sleep(0.1)
-            if await scratch.isAlive():
-                proc.kill()
-            proc.wait()
+            if scratch.uid in self.procs:
+                proc = self.procs[scratch.uid]
+                proc.terminate()
+                for _ in range(10):
+                    if not await scratch.isAlive():
+                        break
+                    await asyncio.sleep(0.1)
+                if await scratch.isAlive():
+                    proc.kill()
+                await proc.wait()
 
         await asyncio.gather(
             *(die_in_piece(scratch) for scratch in self.scratches.values())
         )
 
-    async def load_config(self, config: dict[str, Any]) -> None:
+    async def on_reload(self) -> None:
         "config loader"
-        my_config: dict[str, dict[str, Any]] = config[self.name]
         scratches = {
-            name: Scratch(name, options) for name, options in my_config.items()
+            name: Scratch(name, options) for name, options in self.config.items()
         }
 
         scratches_to_spawn = set()
         for name in scratches:
-            if not self.scratches.get(name):
+            scratch = self.scratches.get(name)
+            if scratch:
+                scratch.conf = scratches[name].conf
+            else:
                 self.scratches.register(scratches[name], name)
                 is_lazy = scratches[name].conf.get("lazy", False)
                 if not is_lazy:
                     scratches_to_spawn.add(name)
-            else:
-                self.scratches.get(name).conf = scratches[name].conf
 
         for name in scratches_to_spawn:
             if await self.ensure_alive(name):
-                self.scratches.get(name).should_hide = True
+                scratch = self.scratches.get(name)
+                assert scratch
+                scratch.should_hide = True
             else:
                 self.log.error("Failure starting %s", name)
 
         for scratch in list(self.scratches.getByState("configured")):
+            assert scratch
             self.scratches.clearState(scratch, "configured")
 
     async def _configure_windowrules(self, scratch):
@@ -466,7 +476,10 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         "Ensure alive, standard version"
         uid = item.uid
         if uid in self.procs:
-            self.procs[uid].kill()
+            try:
+                self.procs[uid].kill()
+            except ProcessLookupError:
+                pass
         self.scratches.reset(item)
         await self.start_scratch_command(uid)
         self.log.info("starting %s", uid)
@@ -475,7 +488,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             if await item.isAlive():
                 error = "The command didn't open a window"
             else:
-                self.procs[uid].communicate()
+                await self.procs[uid].communicate()
                 code = self.procs[uid].returncode
                 if code:
                     error = f"The command failed with code {code}"
@@ -492,6 +505,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         """
         item = self.scratches.get(name=uid)
         await self._configure_windowrules(item)
+        assert item
 
         if item.conf.get("process_tracking", True):
             if not await item.isAlive():
@@ -506,15 +520,10 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
     async def start_scratch_command(self, name: str) -> None:
         "spawns a given scratchpad's process"
         scratch = self.scratches.get(name)
+        assert scratch
         self.scratches.setState(scratch, "respawned")
         old_pid = self.procs[name].pid if name in self.procs else 0
-        proc = subprocess.Popen(
-            scratch.conf["command"],
-            stdin=subprocess.DEVNULL,
-            stdout=subprocess.DEVNULL,
-            stderr=subprocess.DEVNULL,
-            shell=True,
-        )
+        proc = await asyncio.create_subprocess_shell(scratch.conf["command"])
         self.procs[name] = proc
         pid = proc.pid
         scratch.reset(pid)
@@ -546,34 +555,69 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             self.log.info("Didn't update scratch info %s", self)
 
     # Events {{{
+    async def event_monitorremoved(self, monitor_name) -> None:
+        "Hides scratchpads on the removed screen"
+        for scratch in self.scratches.values():
+            if scratch.monitor == monitor_name:
+                await self.run_hide(scratch.uid, autohide=True)
+
     async def event_configreloaded(self, _nothing):
         "Re-apply windowrules when hyprland is restarted"
         for scratch in list(self.scratches.getByState("configured")):
             self.scratches.clearState(scratch, "configured")
             await self._configure_windowrules(scratch)
 
-    async def event_focusedmon(self, mon):
-        "focused monitor hook"
-        self.monitor = mon
-
-    async def event_workspace(self, workspace) -> None:
-        "workspace change hook"
-        self.workspace = workspace
-
     async def event_activewindowv2(self, addr) -> None:
         "active windows hook"
         for uid, scratch in self.scratches.items():
-            if scratch.client_info and scratch.address != addr:
+            if not scratch.client_info:
+                continue
+            if scratch.address == addr:
+                task = self._hysteresis_tasks.get(scratch.uid)
+                if task:
+                    task.cancel()
+                    if scratch.uid in self._hysteresis_tasks:
+                        del self._hysteresis_tasks[scratch.uid]
+                    self.log.debug("Canceled previous task for %s", uid)
+            else:
                 if (
                     scratch.visible
                     and scratch.conf.get("unfocus") == "hide"
                     and not self.scratches.hasState(scratch, "transition")
                 ):
-                    last_shown = self.scratches.get(uid).meta.get("last_shown", 0)
+                    last_shown = scratch.meta.get("last_shown", 0)
                     if last_shown + AFTER_SHOW_INHIBITION > time.time():
                         self.log.debug(
-                            "(SKIPPED) hide %s because another client is active", uid
+                            "(SKIPPED) hide %s because another client is active",
+                            uid,
                         )
+                        continue
+
+                    hysteresis = scratch.conf.get("hysteresis", DEFAULT_HYSTERESIS)
+                    if hysteresis:
+                        task = self._hysteresis_tasks.get(scratch.uid)
+                        if task:
+                            task.cancel()
+                            self.log.debug("Canceled previous task for %s", uid)
+                        self._hysteresis_tasks[scratch.uid] = asyncio.create_task(
+                            asyncio.sleep(hysteresis)
+                        )
+
+                        async def _task(scratch, uid):
+                            await self._hysteresis_tasks[scratch.uid]
+                            del self._hysteresis_tasks[scratch.uid]
+                            if state.active_window == scratch.full_address:
+                                self.log.debug(
+                                    "Skipped hidding %s because client got the focus back",
+                                    uid,
+                                )
+                                return
+                            self.log.debug(
+                                "hide %s because another client is active", uid
+                            )
+                            await self.run_hide(uid, autohide=True)
+
+                        asyncio.create_task(_task(scratch, uid))
                     else:
                         self.log.debug("hide %s because another client is active", uid)
                         await self.run_hide(uid, autohide=True)
@@ -685,9 +729,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             )
             return
 
-        self.focused_window_tracking[uid] = cast(
-            dict[str, Any], await self.hyprctlJSON("activewindow")
-        )
+        self.focused_window_tracking[uid] = state.active_window
 
         if not item:
             self.log.warning("%s is not configured", uid)
@@ -705,6 +747,7 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
             ]
         for e_uid in excluded:
             scratch = self.scratches.get(e_uid)
+            assert scratch
             if scratch.visible:
                 await self.run_hide(e_uid, autohide=True)
         await item.updateClientInfo()
@@ -720,10 +763,11 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         wrkspc = monitor["activeWorkspace"]["id"]
 
         self.scratches.setState(item, "transition")
+        item.monitor = monitor["name"]
         # Start the transition
         await self.hyprctl(
             [
-                f"moveworkspacetomonitor special:scratch_{uid} {monitor['name']}",
+                f"moveworkspacetomonitor special:scratch_{uid} {item.monitor}",
                 f"movetoworkspacesilent {wrkspc},address:{item.full_address}",
                 f"alterzorder top,address:{item.full_address}",
             ]
@@ -797,9 +841,9 @@ class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
         if (
             animation_type and uid in self.focused_window_tracking
         ):  # focus got lost when animating
-            if not autohide and "address" in self.focused_window_tracking[uid]:
+            if not autohide and scratch.conf.get("restore_focus", True):
                 await self.hyprctl(
-                    f"focuswindow address:{self.focused_window_tracking[uid]['address']}"
+                    f"focuswindow address:{self.focused_window_tracking[uid]}"
                 )
                 del self.focused_window_tracking[uid]
 
